import java.awt.*;
import java.awt.event.*;

// Ein Applet zur Spielsteuerung
public class Spielstand /*extends Thread*/ implements ActionListener, SelectListener
	{
	private static final int GAME_END    = 0;	// Spiel beendet
	private static final int GAME_PLAYER = 1;	// Wrfeln
	private static final int GAME_SELECT = 2;	// Spielstein auswhlen
	
	private static final long GAME_SPEED  = 500;	// Wartezeit zwischen Computerzgen
	
	private Wuerfel wrfel;
	private Spielfeld spielfeld;
	
	private int won[] = new int [4];	// Anzahl der Siege
	private int playercount = 1;			// Anzahl der Menschlichen Spieler
	private int current;				// Aktueller Spieler
	private int ntries;					// Versuche, eine 6 zu wrfeln
	private int gamestate = GAME_END;	// Spielstatus
	
	public Spielstand(Spielfeld spielfeld)
		{
		this.spielfeld = spielfeld;
		}
	
	// Statustext und Punkte setzen
	private void updateStatus(String s)
	{
		Color c;
		if (gamestate > GAME_END) c = spielfeld.spieler[current].getColor();
		else c = Color.black;
		spielfeld.setStatus( s, c);
			for (int i = Spielfeld.MIN; i < Spielfeld.MAX; i++){
				int p = (int)(spielfeld.spieler[i].getPoints() * 100) / 170;
				spielfeld.spieler[i].setScore(p);
			}
		}
		
	// Spieler weiterschalten
	private void nextPlayer()
		{
		current++;
		current %= Spielfeld.MAX;

		ntries = 0;
		updateStatus(spielfeld.spieler[current].getName() + " ist dran");
		if (current > playercount) spielfeld.Button("off");
		else spielfeld.Button("on");
		}

	// Spiel initialisieren
	private void newGame()
		{
		this.reset();
		this.gamestate = GAME_PLAYER;
		this.current = -1;

		nextPlayer();
		}	
		
		
	private void reset()
		{
		gamestate = GAME_END;
		updateStatus("Spiel aufgebaut.");		
		spielfeld.reset();
		}
	
	private boolean checkWon()
		{
		if (spielfeld.spieler[current].hasWon())
			{
			won[current]++;
//			siege[current].setText(Integer.toString(won[current]));
			updateStatus(spielfeld.spieler[current].getName() + " gewinnt");
			try{	
				HighScore hs = new HighScore();
				//for (int i = Spielfeld.MIN; i < Spielfeld.MAX; i++){
					hs.writeScore(spielfeld.spieler[0]);
				//}	
			}catch(Exception ex){
				MsgBox.OKDialog("Fehler",ex.getMessage());
			}
			//reset();
			return (true);
			}
		return (false);
		}
	
	private void delayAction()
		{
		Sleeper s = new Sleeper((int)GAME_SPEED, this);
		s.start();
		}
		
	private void sleep(){
		try{
		this.wait(GAME_SPEED);
		}catch(Exception ex){}
	}	

	private void sleep(int i){
		Sleeper s = new Sleeper(i, this);
		s.start();
	}	

	// Button gedrckt
	public void actionPerformed(ActionEvent e)
		{
	//	System.err.println("SP"+current);			
		if ((e != null) && (e.getActionCommand().equals("Neues Spiel")))
			{
			newGame();
			}
		else if ((e != null) && (e.getActionCommand().equals("Wrfeln"))){
			Wuerfeln();
			}
	}
	
	public void Wuerfeln(){

					switch (gamestate)
						{
						case GAME_END:	// Ende
							break;
						
						case GAME_PLAYER:
							if(wrfel!=null)wrfel.visible = false;	
							wrfel = spielfeld.spieler[current].getWuerfel();			
							wrfel.visible = true;
							wrfel.wrfeln();
							int n = wrfel.getNumber();
							//System.err.println(wrfel+"Cur"+current+""+n);
							if (spielfeld.mayRepeat(current))	// Darf dreimal wrfeln?
								{
								if (n != 6)								// Keine 6 - Zhler erhhen
									{
									ntries++;
									if (ntries >= 3){					// 3 Versuche um?
										//System.err.println("Haeh? n="+ntries+" Sp"+current);
										nextPlayer();
									}else{
										updateStatus((ntries + 1) + ". Versuch");
									}
								}else{								// Raussetzen
									PlayerMove(-1, n);
									updateStatus("Nochmal wrfeln");
								}
								
								if (current >= playercount)
									delayAction();
								
								return;
								}
							
							if (n == 6)	// 6 gewrfelt
								{
								int i;
								updateStatus("Nochmal wrfeln");
								
								if (PlayerMove(-1, n))	//Raussetzen
									{
									if (!checkWon() && (current >= playercount))
										delayAction();
								
									return;
									}
								
								i = spielfeld.getStartStone(current);
								if ((i != Spielfeld.KEIN) && PlayerMove(i, n))
									{
									if (!checkWon() && (current >= playercount))
										delayAction();
								
									return;
									}
								}
							
							if (current >= playercount)
								{
								ComputerMove(n);
								if (!checkWon())
									{
									if (n != 6)
										nextPlayer();
									
									if (current >= playercount)
										delayAction();

									}
								}
							else
								{
								if (spielfeld.canMove(current, n))
									{
			//						button.setEnabled(false);
									gamestate = GAME_SELECT;
									spielfeld.setSelectListener(this);
									updateStatus("Spielstein whlen");
									}
								else
									{
									updateStatus(spielfeld.spieler[current].getName() + " kann nicht setzen");
									nextPlayer();
									if (current >= playercount)
										delayAction();

									}
								}
							//sleep(1000);
							break;
						}
	
	}
		
	// Spielstein setzen
	private boolean PlayerMove(int s, int n)
		{
		return (spielfeld.go(current, s, n, false));
		}
	
	// Spielstein automatisch setzen
	private void ComputerMove(int n)
		{
		int s = 0;
		
		//sleep(1000);
		
		// Vordersten Stein suchen
		for (int i = 0; i < 4; i++)
			{
			if (spielfeld.spieler[current].getStonePoints(i) > spielfeld.spieler[current].getStonePoints(s))
				s = i;
			}
		
		if (!PlayerMove(s, n))
			{
			// Kein Zug mglich? Alle Steine probieren
			for (s = 0; s < 4; s++)
				{
				if (PlayerMove(s, n))
					break;
				}
			}
		}
	
	// Wird aufgerufen, wenn der Spieler einen Stein geklickt hat
	public void selected(SelectEvent e)
		{
		int n = wrfel.getNumber();
		
		if (e.getPlayer() != current)	// Falscher Stein
			return;
		if (PlayerMove(e.getStone(), n))
			{
			spielfeld.setSelectListener(null);
//			button.setEnabled(true);
			if (!checkWon())
				{
				gamestate = GAME_PLAYER;
				if (n != 6)
					{
					nextPlayer();
					if (current >= playercount)
						delayAction();
					}
				}
			}
		else
			updateStatus("Zug nicht mglich");
		}

}